#include "sevenseg_display_drv.h"

static sevenseg_display_drv *active7segment = NULL;

sevenseg_display_drv::sevenseg_display_drv() { }

void sevenseg_display_drv::begin_drv()
{
	//Start refreshing Nothing will be displayed before this function called
	//From http://www.instructables.com/id/Arduino-Timer-Interrupts/
	//Set timer1 - 16 bit timer
	
	cli();// disable global interrupts
	TCCR1A = 0;// set entire TCCR1A register to 0
	TCCR1B = 0;// same for TCCR1B
	TCNT1 = 0;//initialize counter value to 0

	//Prescaller (timer speed(Hz)) = (Arduino clock speed(16MHz)) / prescaler
	TCCR1B |= (1 << CS10); //No prescaller 63ns per count for 16MHz clock
	
						   //Also notice how the setups between the three timers differ slightly in the line which turns on CTC mode :
	//TCCR0A |= (1 << WGM01);//for timer0
	//TCCR1B |= (1 << WGM12);//for timer1
	//TCCR2A |= (1 << WGM21);//for timer2
	TCCR1B |= (1 << WGM12);//CTC mode

	// OCR#A (the compare match value)
	//remember that when you use timers 0 and 2 this number must be less than 256, and less than 65536 for timer1
	OCR1A = 999;//intrupt will happen 63ns*[this value] 63usec for 999
	//OCR1A = 496;//31usec
	TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
	sei();//allow interrupts
}

ISR(TIMER1_COMPA_vect) {//change the 1 to 0 for timer0 and 2 for timer2
	active7segment->updateDisplay();//Matrix refrshing function
}

uint8_t sevenseg_display_drv::init()
{
	
	DDRB |= B00111111;//Set pins 8,9,10,11,12,13 as output
	DDRD =  B11111111;//sets pins 0 to 7 as outputs, segments control

	PORTB |= B00111111;//Turn off all digits
	PORTD |= B11111111;//Turn off all segments

	active7segment = this; // For interrupt hander
	buffsize = 4; // According to number of digits
	
	displaybuff = (uint8_t *)calloc(buffsize, sizeof(uint8_t));//allocate for display 
	if (displaybuff == NULL) return 1;

	for (uint8_t n = 0; n < buffsize; n++) {
		dp_buffer[n] = B11111111;//Reset decimal point buffer
		displaybuff[n] = B11111111;//Set all digits to off
	}

	//Map of bits to enable each digit
	digitmap[0] = ~(1 << 0); //Digit 0
	digitmap[1] = ~(1 << 1); //Digit 1
	digitmap[2] = ~(1 << 2); //Digit 2
	digitmap[3] = ~(1 << 3); //Digit 3
	colon_and_ap= B11111111;

	//For pwm_count_max=15 each digit can be turn on up to 1ms out of 8 ms
	//So reftesh rate something like 125Hz
	pwm_count_max = 15;//Number of brightness levels
	brightness = 15;//Default value of brightness

	//Initiate variables
	pwm_count = 0;
	digit_num = 0;
	return 0;
}

void sevenseg_display_drv::updateDisplay()
{
	/*
	This function refreshing display. Called only from interupt
	*/
	
	if (pwm_count > brightness) {
		PORTB |= B00111111;//Turn off all digits PWM brightness control
	}
	
	if (pwm_count < pwm_count_max) pwm_count++;//PWM counter
	else {//End of current pwm cycle
		//Here we need to update information of digit
		pwm_count = 0;//restart pwm counter

		PORTB |= B00111111;//Turn off all digits

		if (digit_num < 3) {// Digits counter 0-3
			digit_num++;//Next digit
			PORTD = displaybuff[digit_num];//Apply data for segments of current digit
			PORTB &= digitmap[digit_num];//Enable relevant digit
		}
		else {//End of current display
			digit_num = 0;//First digit
			PORTD = displaybuff[digit_num];//Apply data for segments of current digit
			PORTB &= digitmap[0]& colon_and_ap;//Colon and apostrophe displayed with first digit
		}
	}
	
	//*** Debug ***
	//long elapsed = micros() - time_start;
	//Serial.println(elapsed);
	//********************
}

//Set value of digit
void sevenseg_display_drv::set_digit( uint8_t dig_num, uint8_t value){
	displaybuff[dig_num]=pgm_read_byte_near(&bcd[value])&dp_buffer[dig_num];
}

//Set brightness of all digits
void sevenseg_display_drv::set_brightness(uint8_t value) {
	if (value<= pwm_count_max) brightness = value;
	else brightness = pwm_count_max;
}

void sevenseg_display_drv::set_dp(uint8_t dig_num) {
	dp_buffer[dig_num] = B01111111;
	displaybuff[dig_num] &= dp_buffer[dig_num];
}

void sevenseg_display_drv::reset_dp(uint8_t dig_num) {
	dp_buffer[dig_num] = B11111111;
	displaybuff[dig_num] &= dp_buffer[dig_num];
}

void sevenseg_display_drv::set_colon() {
	colon_and_ap = colon_and_ap &  ~(1 << 4);
}

void sevenseg_display_drv::reset_colon() {
	colon_and_ap = colon_and_ap | (1 << 4);
}

void sevenseg_display_drv::set_apostrophe() {
	colon_and_ap = colon_and_ap &  ~(1 << 5);
}

void sevenseg_display_drv::reset_apostrophe() {
	colon_and_ap = colon_and_ap | (1 << 5);
}